/*****************************************************************************************
                                          
										   DXPy
							A Simple DirectX9 Wrapper for Python

*****************************************************************************************/
//File: DXPy.cpp
//Use: This is the main file - the program starts here
//By: Daniel Kumor
//Filever: 0.21

/****************************************************************************************/
//All of the imports and includes
/****************************************************************************************/

#include "definitions.h" //Includes the main header
#include "dxpy_module.h" //Includes the dxpy python module
#include "libloader_module.h" //Includes the DLL loader module


/****************************************************************************************/
//The Program's main functions:
/****************************************************************************************/

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
    if( NULL == g_pd3dDevice )
        return;

    // Clear the backbuffer to a blue color
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( BGColor.red, BGColor.green, BGColor.blue ), 1.0f, 0 );

    // Begin the scene
    if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
    {
        // Rendering of scene objects can happen here

		/*!
		I am not sure which would be faster, setting the transform in the
		functions that use it or right here, in every frame...
		It might depend. Since the Camera would be set a lot... right? IDK.
		I will change this around a couple times probably.

		//g_pd3dDevice->SetTransform(D3DTS_VIEW, &(Camera.view));	
		//g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &(Camera.projection));
		*/


		//Loops through Each and every mesh
		for (i=0;Meshes[i].mesh != NULL ; i++) 
		{

			//Is the mesh visible?
			if (Meshes[i].visible!=0) {//Checks to display mesh

				//All of the subsets
				for(DWORD z = 0; z < Meshes[i].nummaterials; z++)    // loop through each subset
				{
					g_pd3dDevice->SetMaterial(&(Meshes[i].material[z]));    // set the material for the subset
					g_pd3dDevice->SetTexture( 0, Meshes[i].texture[z] ); //Sets the Texture

					Meshes[i].mesh->DrawSubset(z); //Draws the Subset
					
				}
			}
		}
		
		// End the scene
		g_pd3dDevice->EndScene();
    }

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}
//-----------------------------------------------------------------------------
// Name: InitDirectInput()
// Desc: Initializes DirectInput 
//-----------------------------------------------------------------------------
HRESULT InitDirectInput() {

	// create the DirectInput interface
    if (FAILED(DirectInput8Create(hInst,    // the handle to the application
                       DIRECTINPUT_VERSION,    // the compatible version
                       IID_IDirectInput8,    // the DirectInput interface version
                       (void**)&Input.din,    // the pointer to the interface
					   NULL))){
		return E_FAIL;
	}
	//Input setup

    // create the keyboard device
    if (FAILED(Input.din->CreateDevice(GUID_SysKeyboard,&Input.keyboard,NULL)))
		return E_FAIL;

    if (FAILED(Input.keyboard->SetDataFormat(&c_dfDIKeyboard)))
		return E_FAIL;

	//Create the Mouse device
	if (FAILED(Input.din->CreateDevice(GUID_SysMouse,&Input.mouse,NULL)))
		return E_FAIL;

	if (FAILED(Input.mouse->SetDataFormat(&c_dfDIMouse)))
		return E_FAIL;

    // set the control you will have over the keyboard
    if (FAILED(Input.keyboard->SetCooperativeLevel(hWnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND)))
		return E_FAIL;

	// set the control you will have over the mouse
    if (FAILED(Input.mouse->SetCooperativeLevel(hWnd,DISCL_NONEXCLUSIVE | DISCL_BACKGROUND)))
		return E_FAIL;

	if (FAILED(Input.mouse->Acquire()))    // Acquire the mouse only once
		return E_FAIL;

	//If none of that failed, return SUCCESS!!!
	return S_OK;
}
//-----------------------------------------------------------------------------
// Name: DoInit()
// Desc: Initializes variables that need to be initialized before runtime.
//-----------------------------------------------------------------------------
void DoInit(){
	for (i=0; i!=KEYMAX ; i++) {
		KeyBind[i].KeyNum = -1;
	}

	//Standard Material
	ZeroMemory(&stdmaterial, sizeof(D3DMATERIAL9));    // clear out the struct for use
	stdmaterial.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);    // set diffuse color to white
    stdmaterial.Ambient = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);    // set ambient color to white
	
	/*
	Sets up preliminary matrices - needed to have successful view
	*/

    //Sets the Camera Look At position
    D3DXMatrixLookAtLH(&Camera.view,
                       &D3DXVECTOR3 (10.0f, 10.0f, 10.0f),    // the camera position
                       &D3DXVECTOR3 (0.0f, 0.0f, 0.0f),    // the look-at position
                       &D3DXVECTOR3 (0.0f, 1.0f, 0.0f));    // the up direction
	


	//lol
}
//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D()
{
    // Create the D3D object, which is needed to create the D3DDevice.
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice. Most parameters are
    // zeroed out. Then we set the SwapEffect to "discard", which is the most
    // efficient method of presenting the back buffer to the display.  And 
    // we request a back buffer format that matches the current desktop display 
    // format (if windowed). Also, since there will be complex geometry visible, 
	// we will use a zbuffer by default.
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof( d3dpp ) );
    
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    
	d3dpp.EnableAutoDepthStencil = TRUE; //Turns on the Z-buffer
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16; //Sets it to 16-bit

	//Dependent on whether the application is Fullscreen or not
	if (G_fullscreen==1) {
		d3dpp.Windowed = FALSE;
		d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8; //Sets the back buffer to 32 bit
		d3dpp.BackBufferWidth = G_width;    // set the width of the buffer
        d3dpp.BackBufferHeight = G_height;    // set the height of the buffer
	}else {
		d3dpp.Windowed = TRUE;
		d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	}
    // Create the Direct3D device. Here we are using the default adapter (most
    // systems only have one, unless they have multiple graphics hardware cards
    // installed) and requesting the HAL (which is saying we want the hardware
    // device rather than a software one). First, we try hardware vertexprocessing
	// which works quickly on all cards. Then, software vertex processing is 
    // specified since we know it will work on all cards. On cards that support 
    // hardware vertex processing, though, we will see a big performance gain 
    // by specifying hardware vertex processing.
	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
	{
		if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
										  D3DCREATE_SOFTWARE_VERTEXPROCESSING,
										  &d3dpp, &g_pd3dDevice ) ) )
		{
			return E_FAIL;
		}
	}

    // Turn off culling
	//EDIT: I dont think culling should be turned off by defalut anymore
    //g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

	// Turn on ambient lighting 
    g_pd3dDevice->SetRenderState( D3DRS_AMBIENT, 0xffffffff );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{

	//Freeing all of the Meshes
	for (i=0; Meshes[i].mesh != NULL; i++)
	{
		//Releases each mesh
		Meshes[i].mesh->Release();

		//Frees each material
		if( Meshes[i].material != NULL )
			delete[] Meshes[i].material;
		
		//Frees all of the textures
		if( Meshes[i].texture )
		{
			for( DWORD z = 0; z < Meshes[i].nummaterials; z++ )
			{
				if( Meshes[i].texture[z] )
					Meshes[i].texture[z]->Release();
			}
			delete[] Meshes[i].texture;
		}
	}	


	//D3D Device
    if( g_pd3dDevice != NULL )
        g_pd3dDevice->Release();

    if( g_pD3D != NULL )
        g_pD3D->Release();

	//DirectInput
	if ( Input.din != NULL )
		Input.din->Release();

	if ( Input.keyboard != NULL )
		Input.keyboard->Release();

	if ( Input.mouse != NULL )
		Input.mouse->Release();

	//Destroy the Window
	if ( hWnd != NULL)
		DestroyWindow(hWnd);
}

//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst_nrml, HINSTANCE, LPSTR, INT )
{
	hInst=hInst_nrml;
	FILE* pythonfile=fopen("main.py","r"); //Opens the python File

	if (pythonfile==NULL) { //Checks if the file was found
		MessageBox(NULL,"DXPy could not open main.py, the program will now fail.",
			"DXPy Error:",MB_OK);
		return 1; //Quits the program abnormally
	}

//#ifdef ISMINGW 
//	//MinGW uses PyObjects to open the file, since it is statically linked
//	fclose(pythonfile); 
//#endif

	fclose(pythonfile); //Closes the file

    // Creates a windows class
	WNDCLASSEX wc =
	{
		sizeof( WNDCLASSEX ), CS_CLASSDC, MsgProc, 0L, 0L,
		GetModuleHandle( NULL ), NULL, NULL,
		NULL, NULL,"DXPy", NULL
	};

	wc.hCursor = LoadCursor(NULL, IDC_ARROW); //Cursor TODO: Remove this?

    RegisterClassEx( &wc ); //Registers the window class
	
	/*Now, we fire up Python!*/
	
	//Adds the module DXPy
	//PyImport_AppendInittab("dxpy", PyInit_dxpy);

	//Initializes the interpreter
	Py_Initialize();

	//Loads the extension modules
	(void) Py_InitModule("dxpy", dxpy_methods);

#ifdef LIBLOADER_MODULE_HEADER //Only tries to load the module if Libloader is included
	(void) Py_InitModule("libloader", libloader_methods);
#endif

//Using Python 3.0 this worked, in python 2.5, this doesnt work.
//#ifdef ISMINGW
//	//MinGW has errors when simplefile is called, so this is a simple workaround
//	PyRun_SimpleString("import main");
//#else
//	//In MSVC, using dynamically linked runtime, you can simply call the file
//	PyRun_SimpleFile(pythonfile,"main.py");
//#endif
	
	DoInit();
	//There was python hell right there, so I just used the workaround
	//Opens, reads and creates a python object from the file
    PyObject* PyFileObject = PyFile_FromString("main.py", "r");
    //Runs the file in the Python interpreter
    PyRun_SimpleFile(PyFile_AsFile(PyFileObject),"main.py");

	//Cleans the PyObject
	Py_DECREF(PyFileObject);

	Py_Finalize(); //Disposes of the interpreter

	//Finishes the program
	Cleanup(); //Calls cleanup
    UnregisterClass( "DXPy", wc.hInstance );
#ifdef LIBLOADER_MODULE_HEADER //Only tries to free the libraries if Libloader is included
	for (i=0; DLL_modules[i].modulename!=NULL; i++)
	{
		if (DLL_modules[i].dll)
			FreeLibrary(DLL_modules[i].dll); //Frees the loaded libraries
		
		//Memory was allocated for modulename, so we have to free it
		//TODO: Free it. I dont feel like finding out WHY this gives an error
		//free((void*)DLL_modules[i].modulename);
	}
#endif
//If it is compiled DEBUG mode, it pauses before quitting, useful in MinGW when there is error output
#if DEBUG 
	system("pause");
#endif

    return 0; //All is well
}
