/*****************************************************************************************
                                          
										   DXPy
							A Simple DirectX9 Wrapper for Python

*****************************************************************************************/
//File: definitions.h
//Use: Here, the program's global variables are defined along with all includes.
//By: Daniel Kumor
//Filever: 0.21

//Sets the Version of DXPy
#define DXPY_VERSION 0.2
//States whether to pause before quitting, useful for debugging
#define DEBUG 1

//Sets the maximum number of meshes that can be used
#define MESHMAX 80
//Sets the maximum number of extension DLLs the application can load
#define DLLMAX 20
//Sets the maximum of keys that can be binded
#define KEYMAX 80
//Sets the maximum amount of fonts that can be used
#define FONTAMT 20
//Sets the maximum amount of text that can be used
#define TXTAMT 40

//Just in case somene tries to compile this without a c++ compiler
#ifndef __cplusplus 
#error This compiler cannot compile c++!
#endif 

//Makes sure that the header is included only once
#ifndef DXPY_HEADER
#define DXPY_HEADER

#include <python.h> //Includes Python

#include <d3d9.h> //Includes Direct3D 9
#include <d3dx9.h>

#include <dinput.h> //Includes DirectInput8

//ISMINGW was defined in the make batch file in MinGW only, the libaries
//are likewise linked in the batch file
#ifndef ISMINGW //Only includes the MSVC libraries if the compiler is NOT MinGW
#pragma comment(lib,"winmm.lib")////
#pragma comment(lib,"d3d9.lib")
#pragma comment(lib,"d3dx9.lib")
#pragma comment(lib,"dinput8.lib")
#pragma comment(lib,"dxguid.lib")
#endif

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

LPDIRECT3D9         g_pD3D = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9   g_pd3dDevice = NULL; // Our rendering device

//EDIT: Changed this to a struct.
//LPDIRECTINPUT8			g_din;    // the pointer to our DirectInput interface
//LPDIRECTINPUTDEVICE8	g_dinkeyboard;    // the pointer to the keyboard device

WNDCLASSEX			wc; //Creates a window class
HWND				hWnd; //Creates a window link
HINSTANCE			hInst; //Creates the HInstance

//Createwindow globals
int					G_width; //the width of window/resolution
int					G_height;//the height of window/resolution
int					G_fullscreen; //Boolean, 1 is fullscreen, 0is normal

//Game Engine binder
PyObject*			g_GameEngineFunction; //The game engine of the game

//Structs!

struct BG_COLOR		{int red, green, blue;};

struct KEY_BINDER	{int KeyNum, CallType, subtype; PyObject* Function;};

//For the mouse, it simply implements each thing, rather than going in a loop
struct MOUSE_BINDER	{PyObject* Left_click, Right_click,
						Left_press,Right_press,
						Left_release,Right_release
						Middle_press,Middle_click
						Middle_release, Mouse_move;
						int L,R,M;}; //LRM are the temp vars to tell if button released

struct CAMERA		{D3DXMATRIXA16 projection; D3DXMATRIX view;};
struct FONTS		{char* FontName; LPD3DXFONT FontType;};
struct TEXTS		{char* TxtName, Content, Color, FontName;};

//Direct Input!
struct DINPUT		{LPDIRECTINPUT8 din; LPDIRECTINPUTDEVICE8 keyboard, mouse;};

//Loads Mesh
struct MESH_LOADER	{char* name; int subsets, visible;
						LPD3DXMESH mesh; D3DMATERIAL9* material;
						LPDIRECT3DTEXTURE9* texture; DWORD nummaterials;};


#define HAS_I
static int			i; //integer i, used for looping


//-----------------------------------------------------------------------------
// Macros
//-----------------------------------------------------------------------------
//Key-press macros
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

//-----------------------------------------------------------------------------
// Global Function Definitions
//-----------------------------------------------------------------------------
VOID Render();
HRESULT InitD3D();
VOID Cleanup();
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT InitDirectInput();

#endif
