/*****************************************************************************************
                                          
										   DXPy
							A Simple DirectX9 Wrapper for Python

*****************************************************************************************/
//!File: dxpy_module.h
//Use: This is the file where the dxpy python module is implemented
//By: Daniel Kumor
//Filever: 0.22

//Makes sure that the header is included only once
#ifndef DXPY_MODULE_HEADER
#define DXPY_MODULE_HEADER

//Checks if i is defined
#ifndef HAS_I
#define HAS_I
static int i;
#endif

//The extension for the module
#include "dxpy_module_EXT.h"

//Checks if maximum key amount is defined
#ifndef MESHMAX
#define MESHMAX 80
#endif

//Checks if maximum key amount is defined
#ifndef KEYMAX
#define KEYMAX 80
#endif

//Checks the maximum font amount
#ifndef FONTAMT
#define FONTAMT 20
#endif

//Checks the maximum text amount
#ifndef TXTAMT
#define TXTAMT 40
#endif

/****************************************************************************************/
//All of the module's global variables:
/****************************************************************************************/

//Boolean, whether the program quits when ESC is pressed
//Defaults to true to lower learning curve
int					G_QuitOnESC		=TRUE;
//Sets the default Background Color
BG_COLOR			BGColor			={0,0,255};
//Sets the possible Key Binds
KEY_BINDER			KeyBind[KEYMAX];
//Sets the Mesh Array
MESH_LOADER			Meshes[MESHMAX];

//Camera Position Matrix
CAMERA Camera;

//DirectInput
DINPUT Input;

//Standard Material
D3DMATERIAL9 stdmaterial;
//Fonts


//States whether there is a frame limit
DWORD				G_FrameLimit	=200;


/****************************************************************************************/
//All of the module's functions:
/****************************************************************************************/

//-----------------------------------------------------------------------------
// Name: createwindow(text,fullscreen?,width/resolution,height/resolution)
// Desc: Initializes the DXPy main window
//-----------------------------------------------------------------------------
PyObject* CreateWindow_py(PyObject *self, PyObject *args)
{
	char* caption;
	if (!PyArg_ParseTuple(args, "siii", &caption, &G_fullscreen, &G_width, &G_height))
        return Py_BuildValue("i",-1);

	// Create the application's window
	if (G_fullscreen==1) {
		hWnd = CreateWindow( "DXPy", caption, //Creates the Window as a popup
                              WS_POPUP, 0, 0, G_width, G_height,
                              NULL, NULL, wc.hInstance, NULL );
	} else {
		hWnd = CreateWindow( "DXPy", caption, //Creates the window as a window...
                              WS_OVERLAPPEDWINDOW, 100, 100, G_width, G_height,
                              NULL, NULL, wc.hInstance, NULL );
	}
    // Initialize Direct3D and DirectInput
    if( FAILED( InitD3D( ) ) || FAILED(InitDirectInput())) //Initializing Direct3D or DirectInput failed
	{
		PostQuitMessage( 0 ); //In case the file still tries  to start the game
		return Py_BuildValue("i",-1); //The user should check if it failed
	}
    // Show the window
    ShowWindow( hWnd, SW_SHOWDEFAULT );
    UpdateWindow( hWnd );
   
	/*
	Final Setup
	*/

	//Sets initial camera perspective (Here because needs window width and height)
	D3DXMatrixPerspectiveFovLH( &(Camera.projection), D3DX_PI / 4,
						(FLOAT)G_width / (FLOAT)G_height, 1.0f, 100.0f );

	//Sets the initial transforms
	//TRANSFORMS
	g_pd3dDevice->SetTransform(D3DTS_VIEW, &(Camera.view));	
	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &(Camera.projection));
	/*
	END of Final Setup
	*/
	
	return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: startgame()
// Desc: Starts the game's main message loop
//-----------------------------------------------------------------------------
PyObject* StartGame_py(PyObject *self, PyObject *args)
{
	MSG msg;
	ZeroMemory( &msg, sizeof( msg ) );
	while( msg.message != WM_QUIT )
	{
		/*
		OK, so here we have the program's main line...this piece of code will be executed 
		about 30 times per second. Here all the movement goes on, the actual drawing of the scene.
		
		This if checks if there are any windows messages, and if not, renders the scene.
		*/
		static DWORD starting_point = GetTickCount();
		if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
			Render(); //In this function all the magic happens
		
		/*Many games will want to quit on escape, so it checks if the escape key is down and
		if it should quit the game. */
		if((KEY_DOWN(VK_ESCAPE)) && (G_QuitOnESC))
            PostQuitMessage( 0 ); //Quits the game if quit on escape is on

		/*This checks if there is a binded gameengine function, and runs it if there is one, 
		skips it if not */
		if (g_GameEngineFunction != NULL) { //Runs the game engine
            static PyObject* arglist=Py_BuildValue("()");
            PyEval_CallObject(g_GameEngineFunction, arglist);
        }

		/* This huge block of code is for the keybinds. The binded keys are checked, and the 
		functions that they are linked to are executed. The code takes into account multiple
		bindings and etc. This allows for basically onclick, buttonpressed and onrelease
		to be easily implemented in DXPy */
		
		//TODO: Capture Mouse
		static DIMOUSESTATE mousestate;    // create a static storage for the mouse-states
		Input.mouse->Acquire();    // get access if we don't have it already
		Input.mouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&mousestate); // fill the mousestate with values

		if(mousestate.rgbButtons[0] & 0x80)    // if the left mouse-button was clicked...

        // then inform the user of this very important message:
        MessageBox(NULL, L"You clicked the mouse button", L"IMPORTANT MESSAGE!", MB_OK);

		//END OF TODO

		//Call Keyboard
		static BYTE keystate[256];    // create a static storage for the key-states
        Input.keyboard->Acquire();    // get access if we don't have it already
        Input.keyboard->GetDeviceState(256, (LPVOID)keystate);    // fill keystate with values

        static int temprunner;
		static PyObject* arglist=Py_BuildValue("()");
		for (temprunner=0; (KeyBind[temprunner].KeyNum != -1) && temprunner!=KEYMAX; temprunner++) {//Checks if binded key was pressed
			if (keystate[KeyBind[temprunner].KeyNum] & 0x80) {
				switch (KeyBind[temprunner].CallType) {
					case 0:
						PyEval_CallObject(KeyBind[temprunner].Function, arglist);
						break;
					case 1:
						if (KeyBind[temprunner].subtype==0) {
							PyEval_CallObject(KeyBind[temprunner].Function, arglist);
							KeyBind[temprunner].subtype=1;
						}
						break;
					case 3:
						KeyBind[temprunner].subtype=1;
						break;
				}
			} else {
				if (KeyBind[temprunner].CallType==0) {
					//To go faster, it checks if it is not 1 first
				} else if (KeyBind[temprunner].CallType==1 && KeyBind[temprunner].subtype==1){
					KeyBind[temprunner].subtype=0;
				}else if (KeyBind[temprunner].CallType==3 && KeyBind[temprunner].subtype==1) {
					KeyBind[temprunner].subtype=0;
					PyEval_CallObject(KeyBind[temprunner].Function, arglist);
				}
			
			}
		}

		while ((GetTickCount() - starting_point) < G_FrameLimit);
	}
	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: loadmesh(string Name,string File,boolean Visible)
// Desc: Loads a mesh from an X file
//-----------------------------------------------------------------------------
PyObject* LoadMesh_py(PyObject *self, PyObject *args)
{
	for (i=0; Meshes[i].mesh != NULL; i++) {}
	char* locn;
	if (!PyArg_ParseTuple(args,"ssi",&(Meshes[i].name), &locn,&(Meshes[i].visible))) { 
        return Py_BuildValue("i", -1);
    }

	if (strcmp(locn,"teapot")==0) {
		
		D3DXCreateTeapot(g_pd3dDevice,&(Meshes[i].mesh),NULL);
		Meshes[i].nummaterials=2;
		Meshes[i].material=new D3DMATERIAL9[2];
		Meshes[i].material[0]=stdmaterial;
		Meshes[i].material[1]=stdmaterial;
		Meshes[i].texture=new LPDIRECT3DTEXTURE9[2];
		Meshes[i].texture[0]=NULL;
		Meshes[i].texture[1]=NULL;
	}
	else 
	{
		LPD3DXBUFFER pD3DXMtrlBuffer; //Temporary buffer material

		if (FAILED(D3DXLoadMeshFromX(locn,    // load this file
						  D3DXMESH_SYSTEMMEM,    // load the mesh into system memory
						  g_pd3dDevice,    // the Direct3D Device
						  NULL,    // we aren't using adjacency
						  &pD3DXMtrlBuffer,    // put the materials here
						  NULL,    // we aren't using effect instances
						  &(Meshes[i].nummaterials),    // the number of materials in this model
						  &(Meshes[i].mesh))))// put the mesh here
		{
			//Failed to find file
			return Py_BuildValue("i",-2);
		}

		// retrieve the pointer to the buffer containing the material information
		D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();

		// create a new material buffer for each material in the mesh
		Meshes[i].material = new D3DMATERIAL9[Meshes[i].nummaterials];
		if( Meshes[i].material == NULL )
			return Py_BuildValue("i",-3); //Out of Memory!

		//Create a new texture for the material
		Meshes[i].texture = new LPDIRECT3DTEXTURE9[Meshes[i].nummaterials];
		if( Meshes[i].texture == NULL )
			return Py_BuildValue("i",-3); //Out of Memory! O noez!

		for(DWORD z = 0; z < Meshes[i].nummaterials; z++)    // for each material...
		{
			//Copy material
			Meshes[i].material[z] = d3dxMaterials[z].MatD3D;    // get the material info
			// Set the ambient color for the material (D3DX does not do this)
			Meshes[i].material[z].Ambient = Meshes[i].material[z].Diffuse; // make ambient the same as diffuse
			
			//Set Texture to NULL
			Meshes[i].texture[z] =NULL;

			//Checks if there is a texture file associated with the mesh
			if( d3dxMaterials[z].pTextureFilename != NULL && strlen( d3dxMaterials[z].pTextureFilename ) > 0 )
			{
				if( FAILED( D3DXCreateTextureFromFileA( g_pd3dDevice,
                                                    d3dxMaterials[z].pTextureFilename,
                                                    &(Meshes[i].texture[z]) ) ) )
				{

					//return Py_BuildValue("i",-4);
					//Rather than failing, it simply acts as if there was no texture
				}
			}

		}
		//Releases Buffer
		pD3DXMtrlBuffer->Release();
	}
	


	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: writetext(string text, string font, integer Xlocation, integer Ylocation)
// Desc: Writes text onto the screen
//-----------------------------------------------------------------------------
PyObject* WriteText_py(PyObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args,"ssii",&(BGColor.red),&(BGColor.green),&(BGColor.blue))) { 
        return Py_BuildValue("i", -1);
    }
	//TODO: THIS!!!!
	printf("\nDXPy :: WriteText: Unimplemented");
	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: bgcolor(red, green, blue)
// Desc: Sets the background color of the game
//-----------------------------------------------------------------------------
PyObject* BGColor_py(PyObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args,"iii",&(BGColor.red),&(BGColor.green),&(BGColor.blue))) { 
        return Py_BuildValue("i", -1);
    }
	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: ispressed(key)
// Desc: Finds out if a key is pressed or not
//-----------------------------------------------------------------------------
PyObject* IsPressed_py(PyObject *self, PyObject *args)
{
	char* tempkey;
	if (!PyArg_ParseTuple(args,"s",&tempkey)) {
        return Py_BuildValue("i", -1);
    }
	BYTE keystate[256];    // create a static storage for the key-states
	Input.keyboard->Acquire();    // get access if we don't have it already
	Input.keyboard->GetDeviceState(256, (LPVOID)keystate);    // fill keystate with values
	int a= GetKeyNumber(tempkey);
	if (a != -1)
		if (keystate[a] & 0x80)
			return Py_BuildValue("i",1);
	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: bindkey(key, bindtype (Bool), function)
// Desc: Binds a key to a function
//-----------------------------------------------------------------------------
PyObject* BindKey_py(PyObject *self, PyObject *args)
{
	char* keyname;
	int keytype;
	PyObject* bindfunction;
	if (!PyArg_ParseTuple(args,"siO:set_callback",&keyname,&keytype,&bindfunction)) {
        return Py_BuildValue("i", -1);
    }
	int thekey = GetKeyNumber(keyname);
	if (thekey == -1)
		return Py_BuildValue("i",-2);

	for (i=0; !(KeyBind[i].KeyNum==thekey && KeyBind[i].CallType==keytype) && i != KEYMAX && KeyBind[i].KeyNum!= -1; i++);
	
	if (i==KEYMAX)
		return Py_BuildValue("i",-3);
	
	//Adds a  python reference to the object
	Py_XINCREF(bindfunction);
	
	//Deref
	Py_XDECREF(KeyBind[i].Function);

	KeyBind[i].KeyNum=thekey;
	KeyBind[i].subtype=0;
	KeyBind[i].CallType=keytype;
	KeyBind[i].Function=bindfunction;

	return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: bindmouse(button, bindtype (Bool), function)
// Desc: Binds the mouse to functions
//-----------------------------------------------------------------------------
PyObject* BindMouse_py(PyObject *self, PyObject *args)
{
	char* keyname;
	int keytype;
	PyObject* bindfunction;
	if (!PyArg_ParseTuple(args,"siO:set_callback",&keyname,&keytype,&bindfunction)) {
        return Py_BuildValue("i", -1);
    }
	if (strcmp(name, "leftbutton")==0 || strcmp(name, "left")==0){
		return 0;
	}
	else if (strcmp(name, "rightbutton")==0 || strcmp(name, "right")==0){
		return 1;
	}
	else if (strcmp(name, "middlebutton")==0 || strcmp(name, "middle")==0){
		return 2;
	}
	else if (strcmp(name, "middlebuttonmove")==0 || strcmp(name, "middlemove")==0){
		return 3;
	}
	else if (strcmp(name, "move")==0 || strcmp(name,"mousemove")){
		return 4;
	}
	else
		return -1;
	if (thekey == -1)
		return Py_BuildValue("i",-2);

	for (i=0; !(KeyBind[i].KeyNum==thekey && KeyBind[i].CallType==keytype) && i != KEYMAX && KeyBind[i].KeyNum!= -1; i++);
	
	if (i==KEYMAX)
		return Py_BuildValue("i",-3);
	
	//Adds a  python reference to the object
	Py_XINCREF(bindfunction);
	
	//Deref
	Py_XDECREF(KeyBind[i].Function);

	KeyBind[i].KeyNum=thekey;
	KeyBind[i].subtype=0;
	KeyBind[i].CallType=keytype;
	KeyBind[i].Function=bindfunction;

	return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: unbindkey(key, bindtype (Bool))
// Desc: UnBinds a key from a function
//-----------------------------------------------------------------------------
PyObject* UnBindKey_py(PyObject *self, PyObject *args)
{
	char* txtname;
	int keytype;
	if (!PyArg_ParseTuple(args,"si",&txtname,&keytype)) {
        return Py_BuildValue("i", -1);
    }
	int thekey = GetKeyNumber(txtname);
	for (i=0; !(KeyBind[i].KeyNum==thekey && KeyBind[i].CallType==keytype) && i != KEYMAX && KeyBind[i].KeyNum!= -1; i++);
	if (KeyBind[i].KeyNum == -1 || i == KEYMAX)
		return Py_BuildValue("i",-2);

	Py_XDECREF(KeyBind[i].Function);
	for (i=i+1; i != KEYMAX && KeyBind[i].KeyNum!= -1; i++){

		KeyBind[i-1].KeyNum=KeyBind[i].KeyNum;
		KeyBind[i-1].subtype=KeyBind[i].subtype;
		KeyBind[i-1].CallType=KeyBind[i].CallType;
		KeyBind[i-1].Function=KeyBind[i].Function;
	}
	KeyBind[i-1].KeyNum= -1;
	return Py_BuildValue("i",0);
}

//-----------------------------------------------------------------------------
// Name: bindgame(function)
// Desc: Binds the main message loop
//-----------------------------------------------------------------------------
PyObject* BindGameEngine_py(PyObject *self, PyObject *args)
{
	if (!PyArg_ParseTuple(args,"O:set_callback",&g_GameEngineFunction)) {
        return Py_BuildValue("i", -1);
    }
	return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: unbindgame(function)
// Desc: unBinds the main message loop
//-----------------------------------------------------------------------------
PyObject* UnBindGameEngine_py(PyObject *self, PyObject *args)
{
	g_GameEngineFunction=NULL;
	return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: screenwidth()
// Desc: Gets the current screen resolution's width
//-----------------------------------------------------------------------------
PyObject* GetScreenWidth_py(PyObject *self, PyObject *args) 
{
   return Py_BuildValue("i",GetSystemMetrics(SM_CXSCREEN));
}
//-----------------------------------------------------------------------------
// Name: screenheight()
// Desc: Gets the current screen resolution's height
//-----------------------------------------------------------------------------
PyObject* GetScreenHeight_py(PyObject *self, PyObject *args)
{
   return Py_BuildValue("i",GetSystemMetrics(SM_CYSCREEN));
}
//-----------------------------------------------------------------------------
// Name: version()
// Desc: Lets the application know what version of DXPy it is using
//-----------------------------------------------------------------------------
PyObject* Version_py(PyObject *self, PyObject *args)
{
    return Py_BuildValue("d", DXPY_VERSION); 
}
//-----------------------------------------------------------------------------
// Name: msgbox(text, caption)
// Desc: A simple message box. This works without initialization.
//-----------------------------------------------------------------------------
PyObject* MsgBox_py(PyObject *self, PyObject *args) //A message box
{
    const char *text;
    const char *top;
    if (!PyArg_ParseTuple(args,"ss",&text,&top)) {
        return Py_BuildValue("i", -1);
        }
    MessageBox(NULL,text,top,MB_OK);
    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: endgame()
// Desc: Sends the quit message to the program's loop
//-----------------------------------------------------------------------------
PyObject* QuitGame_py(PyObject *self, PyObject *args) //A message box
{
    PostQuitMessage( 0 ); //Posts the quit message to the loop
    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: quitonesc(boolean)
// Desc: Sets whether to quit on the escape button
//-----------------------------------------------------------------------------
PyObject* QuitOnESC_py(PyObject *self, PyObject *args) //A message box
{
    if (!PyArg_ParseTuple(args,"i",&G_QuitOnESC)) {
        return Py_BuildValue("i", -1);
        }
    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: UseZBuffer(boolean use Zbuffer)
// Desc: Sets whether to use the Z buffer
//-----------------------------------------------------------------------------
PyObject* UseZBuffer_py(PyObject *self, PyObject *args) //A message box
{
    if (!PyArg_ParseTuple(args,"i",&i)) {
        return Py_BuildValue("i", -1);
        }
    // Turn on/off the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, i );
    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: SetCamera(Xpos,Ypos,Zpos, Xlookat, Ylookat, Zlookat, Xup,Yup,Zup)
// Desc: Sets the Camera's absolute position
//-----------------------------------------------------------------------------
PyObject* SetCamera_py(PyObject *self, PyObject *args) //A message box
{
	float CameraPosition[9];
    if (!PyArg_ParseTuple(args,"fffffffff",&(CameraPosition[0]),&(CameraPosition[1]),&(CameraPosition[2]),//Annoyingly long
                                            &(CameraPosition[3]),&(CameraPosition[4]),&(CameraPosition[5]),
                                            &(CameraPosition[6]),&(CameraPosition[7]),&(CameraPosition[8]))) {
            return Py_BuildValue("i", -1);
        }
        D3DXMatrixLookAtLH(&Camera.view,
                       &D3DXVECTOR3 (CameraPosition[0], CameraPosition[1], CameraPosition[2]),    // the camera position
                       &D3DXVECTOR3 (CameraPosition[3], CameraPosition[4], CameraPosition[5]),    // the look-at position
                       &D3DXVECTOR3 (CameraPosition[6], CameraPosition[7], CameraPosition[8]));    // the up direction
		
		//Sets the transform
		//TRANSFORM
		g_pd3dDevice->SetTransform(D3DTS_VIEW, &(Camera.view));	
		
        return Py_BuildValue("i", 0);
}
//-----------------------------------------------------------------------------
// Name: UseCulling(boolean use)
// Desc: Sets whether to use Culling
//-----------------------------------------------------------------------------
PyObject* UseCulling_py(PyObject *self, PyObject *args) //A message box
{
    if (!PyArg_ParseTuple(args,"i",&i)) {
        return Py_BuildValue("i", -1);
        }
    // Turn on/off culling
    if (i==0) {
    	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
    	}
    else {
    	//TODO: set culling ON?
    	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
    }
    
    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: Use3DLight(boolean use)
// Desc: Sets whether to use Culling
//-----------------------------------------------------------------------------
PyObject* Use3DLight_py(PyObject *self, PyObject *args) //A message box
{
    if (!PyArg_ParseTuple(args,"i",&i)) {
        return Py_BuildValue("i", -1);
        }
    // Turn on/off 3D lights
	printf("DXPy Warning: Use3DLights: Incomplete and not tested. Do not use yet.\n");

    if (i==0) {
    	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, 0xffffffff);
    	}
    else { //If it is something else, then just set it
    	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, 0);
    }
    
    return Py_BuildValue("i",0);
}
/****************************************************************************************/
//The Module's initialization
/****************************************************************************************/

static PyMethodDef dxpy_methods[] = { //Defines all functions in the module

//TODO: Weed out the unneeded versions of commands to create a standard command for everything

	{"createwindow", CreateWindow_py, METH_VARARGS, "Initializes the game window"},

	{"startgame", StartGame_py, METH_VARARGS, "Starts the game's main message loop"}, //?Which better?
	{"startloop", StartGame_py, METH_VARARGS, "Starts the game's main message loop"},

	{"getscreenwidth", GetScreenWidth_py, METH_VARARGS, "Gets the current screen resolution's width"}, //?Which better?
	{"getscreenheight", GetScreenHeight_py, METH_VARARGS, "Gets the current screen resolution's height"}, //?Which better?
	{"screenwidth", GetScreenWidth_py, METH_VARARGS, "Gets the current screen resolution's width"},
	{"screenheight", GetScreenHeight_py, METH_VARARGS, "Gets the current screen resolution's height"},
	
	{"bindkey", BindKey_py, METH_VARARGS, "Binds a key to a function"},
	{"unbindkey", UnBindKey_py, METH_VARARGS, "UnBinds a key from a function"},

	{"ispressed", IsPressed_py, METH_VARARGS, "Checks if a key is pressed"},
	{"getkeystate", IsPressed_py, METH_VARARGS, "Checks if a key is pressed"},
	{"getkey", IsPressed_py, METH_VARARGS, "Checks if a key is pressed"},

	{"bindgame", BindGameEngine_py, METH_VARARGS, "Binds the game to a function"},
	{"bindgameengine", BindGameEngine_py, METH_VARARGS, "Binds the game to a function"},
	{"mainfunction", BindGameEngine_py, METH_VARARGS, "Binds the game to a function"},

	{"unbindmainfunction", UnBindGameEngine_py, METH_VARARGS, "UnBinds the game to a function"},
	{"unbindgame", UnBindGameEngine_py, METH_VARARGS, "UnBinds the game to a function"},
	{"unbindgameengine", UnBindGameEngine_py, METH_VARARGS, "UnBinds the game to a function"},

	{"bgcolor", BGColor_py, METH_VARARGS, "Sets the background color RGB mode"},

	{"version", Version_py, METH_VARARGS, "Gets DXPy's version"}, //?which better?
	{"dxpy_version", Version_py, METH_VARARGS, "Gets DXPy's version"},

	{"quitonesc", QuitOnESC_py, METH_VARARGS, "Sets whether to end the application with the escape key"},

	{"endgame", QuitGame_py, METH_VARARGS, "Ends the application"},
	{"quitgame", QuitGame_py, METH_VARARGS, "Ends the application"},
	{"quit", QuitGame_py, METH_VARARGS, "Ends the application"},

	{"messagebox", MsgBox_py, METH_VARARGS, "Shows a simple message box"}, //? which better
	{"msgbox", MsgBox_py, METH_VARARGS, "Shows a simple message box"},     //??

	{"usezbuffer", UseZBuffer_py, METH_VARARGS, "Decides whether to use the Z buffer in the application"},
	{"useculling", UseCulling_py, METH_VARARGS, "Decides whether to use Culling or not"},
	{"use3dlight", Use3DLight_py, METH_VARARGS, "Decides whether to use 3D lighting or not"},
	
	{"setcamera", SetCamera_py, METH_VARARGS, "Sets the camera's absolute position"},

	{"setmesh", LoadMesh_py, METH_VARARGS, "Loads a mesh from an X file"},
	
	{NULL,NULL,0,NULL}  /* Sentinel */
};

//Python 3.0 is NOT used anymore
//static struct PyModuleDef dxpy_module = { //Defines the module
//   PyModuleDef_HEAD_INIT,
//   "dxpy",   /* name of module */
//   NULL, /* module documentation, may be NULL */
//   -1,       /* size of per-interpreter state of the module,
//                or -1 if the module keeps state in global variables. */
//   dxpy_methods
//};
//PyMODINIT_FUNC PyInit_dxpy(void) //Gets the module ready to embed into the interpreter
//{
//    return PyModule_Create(&dxpy_module);
//}


#endif
