/*****************************************************************************************

										   DXPy
							A Simple DirectX9 Wrapper for Python

*****************************************************************************************/
//File: libloader.h
//Use: This allows the loading of expansion modules to DXPy as DLLs
//By: Daniel Kumor
//Filever: 0.10

//Makes sure that the header is included only once
#ifndef LIBLOADER_MODULE_HEADER
#define LIBLOADER_MODULE_HEADER

//Checks if i is defined
#ifndef HAS_I
#define HAS_I
static int i;
#endif

//Checks if maximum library amount is defined
#ifndef DLLMAX
#define DLLMAX 20
#endif

//This is an array of modules
struct DLLMETHOD {PyMethodDef* module; char* modulename; HINSTANCE dll;};
//I do not think that more than 20 libraries will ever be used at once...
DLLMETHOD DLL_modules[DLLMAX];
/****************************************************************************************/
//All of the module's functions:
/****************************************************************************************/

//-----------------------------------------------------------------------------
// Name: load_DLL(dllname, modulename)
// Desc: A simple message box
//-----------------------------------------------------------------------------
PyObject* LoadDLL_py(PyObject *self, PyObject *args)
{
    char *filename;
	char *modulename;
    if (!PyArg_ParseTuple(args,"ss",&filename, &modulename)) {
        return Py_BuildValue("i", -1);
        }

	//Checks if the user tried to get the modules named the same way
	if (strcmp(modulename,"dxpy")==0 || strcmp(modulename,"libloader")==0)
		return Py_BuildValue("i",-2);

	//Finds the next free spot for a DLL
	for (i=0; DLL_modules[i].modulename!=NULL && i!=DLLMAX; i++)
	{
		if (strcmp(DLL_modules[i].modulename,modulename)==0)
			return Py_BuildValue("i", -2); //If such a module exists already, return -2
	}
	if (i==DLLMAX) { //Maximum DLL amount has been achieved
		return Py_BuildValue("i", -6);
	}
	//Tries to load the library, if it fails, then returns -3
	if (!(DLL_modules[i].dll = LoadLibrary(filename)))
		return Py_BuildValue("i", -3);

	//Tries to load the module_methods of the DLL, if fails loading returns -4
	if (!(DLL_modules[i].module=(PyMethodDef*)GetProcAddress(DLL_modules[i].dll,"module_methods")))
	{
		FreeLibrary(DLL_modules[i].dll);
		return Py_BuildValue("i", -4);
	}

	try {
	//Initializes the module. Can be an error if module_methods is messed up
		(void) Py_InitModule(modulename, (DLL_modules[i].module));
	}
	catch(char*)
	{
		FreeLibrary(DLL_modules[i].dll);
		return Py_BuildValue("i", -5);
	}

	//Allocates memory for the modulename string
	/*I know I should use New, but who cares?*/
	DLL_modules[i].modulename=(char*)malloc(sizeof(char)*strlen(modulename));
	//Copies the modulename string
	strcpy(DLL_modules[i].modulename,modulename);

    return Py_BuildValue("i",0);
}
//-----------------------------------------------------------------------------
// Name: unload_DLL(dllname)
// Desc: Unloads a Library
//-----------------------------------------------------------------------------
PyObject* UnloadDLL_py(PyObject *self, PyObject *args)
{
	/*TODO: implement this perhaps? Don't know if it is needed though,
		since all of the libraries are unloaded when the program finishes
	*/
	
	//Returns -99 whenever an unimplemented method is called
	return Py_BuildValue("i",-99);
	
}
/****************************************************************************************/
//The Module's initialization
/****************************************************************************************/
static PyMethodDef libloader_methods[] = { //Defines all functions in the module
	{"load_DLL", LoadDLL_py, METH_VARARGS, "Loads a DLL from file"},
	{NULL,NULL,0,NULL}  /* Sentinel */
};
#endif
