#! /usr/bin/python
"""
	The above line is for linux environments.

    NOTE: This version of DXPY is adapted to work on Python 3.x.
    It will not work on python 2.x
    
    The 2 lines the average user edits are the location of the folder
    on the hard drive:
        hdd=<location>
    and the location of the folder on the flash drive
        flash=<location>

    NOTE: a '\\' is needed in place of the '\' in both locations
        EX:
            hdd="C:\\Folder\\folder"

There are 4 lines of input. Modify only the two that correspond with your
operating system. This was made if you want to have this program work
for a dual boot computer with windows and linux.

These are the four locations. Modify them to work for you:
"""

#If you are working on Windows:

#The location of the folder on the Hard Drive
hdd="C:\\programming\\DXPy" 
#The location of the folder on the Flash Drive
flash="E:\\DXPy"

#If you are working on linux:

#The location of the folder on the Filesystem
lhdd="/home/daniel/Desktop/prgmming"
#The location of the folder on the flash drive
lflash="/media/DANIEL/prgmming"

"""
Made by: Daniel Kumor (2008)
Mod: 2009 (Py3.0)

Commands:
    -----------------------------------------------------------------
    e   - All files and directories that exist in one location,
        but not the other are copied, so that both directories
        have the same files. Then, the program searches the times
        all files were last modified, and keeps the files modified
        most recently, putting them in both directories.
    h   - The end result is that the Flash drive has a copy of the
        directory on the Hard drive. First, it checks the flash drive
        for files that exist in the flash drive but not on the HD.
        Since the desired result is a copy of the HD directory, the
        files are deleted. Then it checks for files that exist on the HD
        but not on the flash drive. It copies the files to the flash drive.
        Lastly, it checks for modified files. It searches the times the
        files were modified, and if they are not equal, the HD version always
        dominates.
    f   - Same as h, except the master is the flash drive, therefore the
        end result is the HD having a copy of the folder of the flash drive
    t   - This is simple. It deletes the directory on the flash drive and copies
        the directory on the Hard drive to the flash drive. Might take a loong
        time for large folders.
    x   - Same as t, except the other way around, deletes from Hard drive,
        copies from flash drive.
    z   - Exits the program


The program might take quite a bit of time, especially if the directories
are large, or there are a lot of files to update.

TODO: Make it MUCH faster. I have no idea how to do it right now, but I am
working on it. The main problem seems to be the 'os.path.isdir' which is used
in all four loops. I tried speeding it up by simply checking if the filename
has a period, and the program then took about 10% of the time it currently takes
to execute, BUT it had an error when directories with periods were part of the test

If someone feels like doing it, they could try implementing the os.walk()
function to do all of the actions, which would probably make it much faster,
but I do NOT feel like messing around with it right now. My method works.
That is all I have to know. It takes 15 seconds to check 100MB
(about 1700 files). I can wait. But for all of you that want to sync 4GB in
10 seconds... all I can say is good luck. Feel free to mess around with my
code.

I added a ton of comments, because I was bored.

Please note that I define all functions at the beginning, and the starter code
is at the end. It might be easier to understand if the starter code is read
first.
"""




#Do not modify beyond this location

#Unless, of course, you know what you are doing

import shutil, os, sys

#This function is always called on error.
def err():
    print(" ")
    print("There was an error. Sync cannot continue:")
    print(sys.exc_info()[1])
    print(" ")
    #Simplest way to handle error: make user fix it.
    print("Try to fix it, and sync again.")
    input("<<Press enter to exit>>")
    sys.exit()
    
 
def copy(master, slave):
    
    #Gets all of the files in the master and slave directories
    mstrfiles=os.listdir(master)
    slavefiles=os.listdir(slave)
    
    for files in mstrfiles: #For each file in the master directory
        if slavefiles.count(files)==0: #If it does not exist in the slave directory
            print("Copying:",os.path.join(master,files),)
            if os.path.isdir(os.path.join(master,files))==True: #Checks which copy to use
                #Copies the directory
                shutil.copytree(os.path.join(master,files), os.path.join(slave,files))
            else:
                #Copies the file
                shutil.copy2(os.path.join(master,files), os.path.join(slave,files))
            print("                                                                    Done")
        else: #if it DOES exist
            if os.path.isdir(os.path.join(master,files))==True: #checks if it is a directory
                #Checks the subdirectory
                copy(os.path.join(master,files), os.path.join(slave,files)) 
        
def delete(master, slave):
    #Gets all of the files in the master and slave directories
    mstrfiles=os.listdir(master)
    slavefiles=os.listdir(slave)

    for files in slavefiles: 
        if mstrfiles.count(files)==0: #If it does not exist in the master directory
            print("Deleting:",os.path.join(slave,files),)
            if os.path.isdir(os.path.join(slave,files))==True: #Checks which delete to use
                #Deletes the directory, and all subdirectories
                shutil.rmtree(os.path.join(slave,files)) 
            else:
                #Deletes the file
                os.remove(os.path.join(slave,files))
            print("                                                                    Done")
        else: #If the file DOES exist
            if os.path.isdir(os.path.join(slave,files))==True: #Checks if it is a directory
                #checks the subdirectory
                delete(os.path.join(master,files), os.path.join(slave,files))
        
    
def updateslv(master, slave): #Can only be used when directories are the same
    for files in os.listdir(master):
        #Checks if it is a directory (dir mod times are not needed)
        if os.path.isdir(os.path.join(master,files))==False:
            #Get stat info about the 2 files
            p=os.stat(os.path.join(master,files))
            o=os.stat(os.path.join(slave,files))

            """
            The next two lines are because Flash drives usually use the FAT32 filesystem,
            While Hard drives use NTFS. --> FAT32 saves modification times accurate to
            2 seconds, while others to 1 second. This makes sure that any modification time
            within 2 seconds will be accepted
            """
            l=p.st_mtime
            l=[l+2,l+1,l,l-1,l-2]
            
            if l.count(o.st_mtime)==0: #Checks if modification times are equal
                print("Updating:", os.path.join(slave,files),)
                #For simplicity, it deletes the file, then copies it from master to slave
                os.remove(os.path.join(slave,files))
                shutil.copy2(os.path.join(master,files), os.path.join(slave,files))
                print("                                                                    Done")
        else:
            #Checks the subdirectory
            updateslv(os.path.join(master,files), os.path.join(slave,files))
        
                
def updateeq(master, slave): #Can only be used when directories are the same
    for files in os.listdir(master):
        if os.path.isdir(os.path.join(master,files))==False:
                p=os.stat(os.path.join(master,files))
                o=os.stat(os.path.join(slave,files))

                #See the function above
                l=p.st_mtime
                l=[l+2,l+1,l,l-1,l-2]
                
                if l.count(o.st_mtime)==0: #Checks if mod times are equal
                    if p.st_mtime>o.st_mtime: #Checks which file is the newer one
                        print("Updating:", os.path.join(slave,files),)
                        os.remove(os.path.join(slave,files))
                        shutil.copy2(os.path.join(master,files),os.path.join(slave,files))
                    else:
                        print("Updating:", os.path.join(master,files),)
                        os.remove(os.path.join(master,files))
                        shutil.copy2(os.path.join(slave,files), os.path.join(master,files))
                    print("                                                                    Done")
        else:
            #Checks the subdirectory
            updateeq(os.path.join(master,files), os.path.join(slave,files))
            
#checks if linux or Windows
if sys.platform != "win32":
	hdd=lhdd
	flash=lflash
#Before the program starts, it checks if both directories exist
if os.path.exists(hdd)==False or os.path.exists(flash)==False: 
    if os.path.exists(hdd)==False:
        print("The folder:", hdd, "could not be found.")
    else:
        print("The folder:",flash, "could not be found.")
    print(""" Please make sure that you have your flash drive
plugged in, and that you have all needed write priveleges""")
    input("<<Please press enter if you connected everything >>")
    while os.path.exists(hdd)==False or os.path.exists(flash)==False:
        print("Nope... Could not find it.")
        input("<<Press enter to try again>>")
    print("\n\n\n")


#The starter code
print("""Made by: Daniel Kumor. To change directories, open the code
and follow instructions.Before you begin, make sure that you are not
using any of the files in either folder needed:""")
print("Folder on Hard Drive:", hdd)
print("Folder on Flash Drive:", flash)
print("""Commands:
    e -Files that exist on the HD but not on the
        flash are created on flash, and vice versa.
        modification saves are based on the most recent one
    h -HDD is master, changes only differences between
        flash drive and HD, until flash is equal to HD
    f -Same as h, except Flash is the master, and it
        makes the HD folder equal to the flash folder
    t -total, deletes on Flash, copies from HD to Flash
    x -Same as t, but deletes on HD, copies from Flash to HD
    z -Exits Program
""")

os.stat_float_times(False)

a=input("Sync: ")
while a!="e" and a!="h" and a!="f" and a!="t" and a!="x" and a!="z":
    print("Command not recognized. Please try again")
    a=input("Sync: ")
if a!="z":
    print("Sync started... (" + a + ")")
    print(" ")
if a=="t":
    try:
        print("Deleting from Flash Drive...")
        #Deletes the directory at flash
        shutil.rmtree(flash)
        print("Done Deleting from Flash Drive")
        print("Copying from Hard Drive...")
        #Copies the new directory to the location of the old one
        shutil.copytree(hdd,flash)
        print("Done copying from HD")
    except:
        err()
elif a=="x":
    try:
        print("Deleting from Hard Drive...")
        shutil.rmtree(hdd)
        print("Done Deleting from Hard Drive")
        print ("Copying from Flash Drive...")
        shutil.copytree(flash,hdd)
        print("Done copying from Flash Drive")
    except:
        err()
elif a=="h":
    try:
        #Deletes all files that do not exist on the master
        delete(hdd, flash)
        #Copies all files that do not exist on slave
        copy(hdd, flash)
        #Updates files that were modified to the master version
        updateslv(hdd, flash)
    except:
        err()
elif a=="f":
    try:
        delete(flash, hdd)
        copy(flash, hdd)
        updateslv(flash, hdd)
    except:
        err()
elif a=='e':
    try:
        #Copies files that are in one, but not the other
        copy(hdd, flash)
        #Then vice versa
        copy(flash, hdd)
        #Updates files that were modified to the newest version
        updateeq(flash, hdd)
    except:
        err()
else:  #Only z is left: exit the program
    sys.exit()

print(" ")
print('Done with sync ('+a+')')
input("<< Press Enter to exit >>")
